#= require jquery
#= require bootstrap/modal

window.LM ||= {}

LM.MODAL_CONTENT_LOADED = 'lm.modal.content.loaded'

LM.loadRemoteModalByID = (modalID, modalOptions={}) ->
  lazyModal = new LazyModalModal(modalID, modalOptions)
  lazyModal.show()
  lazyModal

LM.triggerLoadRemoteModal = (event) ->
  event.preventDefault()
  $target = $(event.currentTarget)
  $target = $(event.currentTarget)
  modalOptions = $target.data('modal-options') || {}
  modalID = _parseModalIDFromHref($target) || modalOptions.target || modalOptions.id
  LM.loadRemoteModalByID modalID, modalOptions

_parseModalIDFromHref = ($target)->
  href = $target.attr('href')
  return href if href != 'javascript:void(0)' and href != '#' and href
  return null

class LazyModalModal
  constructor: (@originModalID, @modalOptions) ->
    @modalOptions.reload ||= false #force load remote modal
    @modalTitle = @modalOptions?.default_modal?.title || ''
    alert('you must set modal ID in dom with data-target or attributes href') unless @originModalID
    if @originModalID.indexOf('#') !=-1
      @modalID = @originModalID?.split("#").pop()
    else
      @modalID = @originModalID
      @originModalID = "##{@originModalID}"

  show: ->
    return unless @modalID
    @_findOrCreateModal()
    @_showModal()

  _findOrCreateModal: ->
    if $(@originModalID).length
      @modal = $(@originModalID)
    else
      modalDom = @_createModalDom()
      @modal = $(modalDom)
      @_addModalEventListener()

  _showModal: ->
    @modal.on 'hidden.bs.modal', =>
      if @modalOptions.reload
        @modal.remove()
        @modal = null

    @modal?.modal 'show' unless @modal?.isShown

  _createModalDom: ->
    """
      <div class="modal light dialog in" tabindex="-1" role="dialog" aria-hidden="true">
        <div class="modal-dialog #{@modalOptions?.default_modal?.size || ''}">
	        <div class="modal-content">
	          <div class="modal-header">
	            <button type="button" class="close" data-dismiss="modal"><span>×</span></button>
	            <h4 class="modal-title">#{@modalTitle || ''}</h4>
	          </div>
	          <div class="modal-body clearfix">
	            <h5 class="text-center">loading…</h5>
	          </div>
	        </div>
        </div>
      </div>
    """

  _addModalEventListener: ->
    @modal.on 'shown.bs.modal', =>
      return if $("##{@modalID}").length
      data = {modal_dir: @modalOptions?.dir}
      $.get("/lazy_modals/#{@modalID}", data)
        .done (response) => @_loadRemoteModalSuccessed response
        .fail (response, xhr, status) =>
          @modal.find('.modal-body').html response.responseText

  _loadRemoteModalSuccessed: (response)->
    @modal?.modal 'hide'
    @modal = $(response)
    $('body').append @modal
    @modal.trigger LM.MODAL_CONTENT_LOADED, @modalOptions
    @show()

$(document).on 'click', '[data-lazy-modal="true"]', LM.triggerLoadRemoteModal
